import openai
import json
import requests
from typing import List, Dict
from retry.api import retry_call
from openai.error import RateLimitError
from openai.error import APIError
from openai.error import TryAgain
from openai.error import Timeout
from openai.error import APIConnectionError
from openai.error import ServiceUnavailableError
from openai.error import InvalidRequestError
from key import API_KEY
import argparse



DEFAULT_API_MAX_RETRIES = 3
DEFAULT_API_DELAY_TIME = 5
API_RETRY_EXCEPTIONS = (
    requests.HTTPError,
    RateLimitError,
    APIError,
    TryAgain,
    Timeout,
    APIConnectionError,
    ServiceUnavailableError)
API_CATCH_EXCEPTIONS = (
    requests.HTTPError,
    RateLimitError,
    APIError,
    TryAgain,
    Timeout,
    APIConnectionError,
    ServiceUnavailableError,
    InvalidRequestError)


def send_prompt_with_context(model: str,
                             messages: List[Dict],
                             temperature: int = 0.1,
                             max_tokens: int = 500) -> Dict[str, str]:
    openai.api_key = API_KEY
    resp = openai.ChatCompletion.create(
        messages=messages,
        model=model,
        temperature=temperature,
        max_tokens=max_tokens
    )
    print(resp)
    return resp.choices[0].message["content"]


def predict_domain(model: str, input_text: str):
    def predict_domain() -> Dict[str, str]:

        system_msg = """
        Determine which domain is considered in the following dialogue situation.
        Choose exactly one domain from this list: restaurant, hotel, attraction, taxi, train
        Answer with only one word, the selected domain from the list. You have to always select the most probable domain.
        -------- Example 1: --------
        Customer: I need a cheap place to eat
        Assistant: We have several not expensive places available. What food are you interested in?
        Customer: Chinese food. 
        Domain: restaurant
        -------- Example 2: --------
        Customer: What is the address?
        Assistant: It's 123 Northfolk Road. 
        Customer: That's all. I also need a train from London.
        Domain: train
        """


        messages = [
            {"role": "system", "content": system_msg},
            {"role": "user", "content": f"Customer: {input_text}"}
        ]

        answer = send_prompt_with_context(model, messages)
        print(answer)
        return answer
    return retry_call(predict_domain,
                      exceptions=API_RETRY_EXCEPTIONS,
                      delay=DEFAULT_API_DELAY_TIME,
                      tries=DEFAULT_API_MAX_RETRIES)
if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="OpenAI API Wrapper for Few-Shot Learning.")
    parser.add_argument("-i", "--input", type=str, default="I am looking for a cheap place to stay.")
    args = parser.parse_args()
    model = "gpt-3.5-turbo"
    answer = predict_domain(model, args.input)